// Copyright (c) 2022 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#pragma once

//------------------------------------------------------------------------------

#include "TCPConnector.h"

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl { namespace io {

//------------------------------------------------------------------------------

/**
 * Client socket that connects to a TCP server socket.
 *
 * The server socket's address and port should be given.
 */
class TCPClientSocket : public TCPSocket
{
public:
    /**
     * Convert the given string to an address
     */
    static struct in_addr str2address(const char* addressStr);

private:
    /**
     * The address to use for connecting.
     */
    TCPConnector connector;

public:
    /**
     * Construct the client socket with the given address and port.
     *
     * @param waiter the waiter to use for waiting a possible event
     * occuring on the socket. If given, the socket will be
     * non-blocking. Otherwise, it will be blocking, and the connect()
     * function of the Connector might block as well as the reading
     * and writing buffers.
     */
    TCPClientSocket(struct in_addr address, unsigned short port,
                    Waiter* waiter = 0,
                    size_t readingCapacity = DEFAULT_CAPACITY,
                    size_t writingCapacity = DEFAULT_CAPACITY);

    /**
     * Construct the client socket with the given address and port.
     *
     * @param waiter the waiter to use for waiting a possible event
     * occuring on the socket. If given, the socket will be
     * non-blocking. Otherwise, it will be blocking, and the connect()
     * function of the Connector might block as well as the reading
     * and writing buffers.
     */
    TCPClientSocket(const char* address, unsigned short port,
                    Waiter* waiter = 0,
                    size_t readingCapacity = DEFAULT_CAPACITY,
                    size_t writingCapacity = DEFAULT_CAPACITY);

    /**
     * Get the connector for this client.
     */
    TCPConnector& getConnector();

    friend class TCPConnector;
};


//------------------------------------------------------------------------------
// Inline definitions
//------------------------------------------------------------------------------

inline TCPClientSocket::TCPClientSocket(const char* address, unsigned short port,
                                        Waiter* waiter,
                                        size_t readingCapacity,
                                        size_t writingCapacity) :
    TCPClientSocket(str2address(address), port, waiter,
                    readingCapacity, writingCapacity)
{
}

//------------------------------------------------------------------------------

inline TCPConnector& TCPClientSocket::getConnector()
{
    return connector;
}

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::scpl::io */ } /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
